package org.jpbm.exploratory;

import java.sql.SQLException;
import java.util.List;

import org.jbpm.JbpmConfiguration;
import org.jbpm.JbpmContext;
import org.jbpm.command.CommandService;
import org.jbpm.command.StartProcessInstanceCommand;
import org.jbpm.command.impl.CommandServiceImpl;
import org.jbpm.db.GraphSession;
import org.jbpm.graph.def.ProcessDefinition;
import org.jbpm.graph.exe.ProcessInstance;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

@Test
public class TestMultiSignal {

	private static final int SIGNALIST_COUNT = 20;
	public static final String PROCESS_NAME = "TestProcess";
	private static JbpmConfiguration jbpmConfiguration = null;
	public static CommandService commandService = null;
	public static long tokenId = -1;
	Signalist[] signalists = new Signalist[SIGNALIST_COUNT];
	public static final String PROCESS_DEFINITION = "<?xml version='1.0' encoding='UTF-8'?>"
			+ "<process-definition xmlns='' name='TestProcess'>"
			+ "<start-state name='start-state1'>"
			+ "<transition to='Service 1'></transition>"
			+ "</start-state>"
			+ "<node name='Service 1'>"
			+ "<action name='esbAction' "
			+ "class='org.jpbm.exploratory.SimpleAction'>"
			+ "</action>"
			+ "<transition to='Service 2'></transition>"
			+ "</node>"
			+ "<node name='Service 2'>"
			+ "<action name='esbAction' "
			+ "class='org.jpbm.exploratory.SimpleAction2'>"
			+ "</action>"
			+ "<transition to='end-state1'></transition>"
			+ "</node>"
//			+ "<node name='Service 3'>"
//			+ "<action name='esbAction' "
//			+ "class='org.jpbm.exploratory.SimpleAction3'>"
//			+ "</action>"
//			+ "<transition to='end-state1'></transition>"
//			+ "</node>"
			+ "<end-state name='end-state1'></end-state>"
			+ "</process-definition>";

	@BeforeClass
	public void setUp() throws SQLException {
		jbpmConfiguration = JbpmConfiguration.getInstance();
		commandService = new CommandServiceImpl(jbpmConfiguration);
		JbpmContext jbpmContext = jbpmConfiguration.createJbpmContext();
		GraphSession gs = jbpmContext.getGraphSession();
		@SuppressWarnings("unchecked")
		List<ProcessDefinition> processDefinitions = (List<ProcessDefinition>) gs
				.findAllProcessDefinitions();
		for (ProcessDefinition pd : processDefinitions) {
			gs.deleteProcessDefinition(pd);
		}
		jbpmContext.deployProcessDefinition(ProcessDefinition
				.parseXmlString(PROCESS_DEFINITION));
		System.out.println("Isol " + jbpmContext.getConnection().getTransactionIsolation());
		jbpmContext.close();
	}

	@Test
	public void testMultipleSignals() {
		StartProcessInstanceCommand startCommand = new StartProcessInstanceCommand();
		startCommand.setProcessName(PROCESS_NAME);
		ProcessInstance pi = (ProcessInstance)commandService.execute(startCommand);
		tokenId = pi.getRootToken().getId();
//		ProcessInstance pi = jbpmContext.newProcessInstance();
//		jbpmContext.save(pi);
//		pi.signal();
//		long t = pi.getRootToken().getId();
//		System.out.println(t);
//		jbpmContext.save(pi);
//		jbpmContext.close();
//		jbpmConfiguration.close();
		for (int i = 0; i < signalists.length; i++) {
			signalists[i] = new Signalist();
			signalists[i].setName("Signalist " + i);
			signalists[i].start();
		}
		// try {
		// Thread.sleep(1000);
		// } catch (InterruptedException e) {
		// }
//		tokenId = t;
		// jbpmContext.getToken(tokenId).signal();
		// pi.signal();
		// System.out.println(pi.hasEnded());

	}

	@AfterClass
	public void tearDown() throws InterruptedException {
		for (int i = signalists.length - 1; i >= 0; i--) {
			signalists[i].join();
			// signalists[i].close();
		}
		System.out.println("Success " + Signalist.successCount);
		System.out.println("Failure " + Signalist.failCount);
	}

	public static void main(String[] arg) throws Exception {
		TestMultiSignal t = new TestMultiSignal();
		t.setUp();
		t.testMultipleSignals();
		t.tearDown();
	}
}
